package com.cultural.heritage.service.book.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.cultural.heritage.common.ErrorCode;
import com.cultural.heritage.exception.BusinessException;
import com.cultural.heritage.exception.ThrowUtils;
import com.cultural.heritage.mapper.ClothesGradeMapper;
import com.cultural.heritage.model.entity.ClothesGrade;
import com.cultural.heritage.model.entity.ClothesInfo;
import com.cultural.heritage.service.book.ClothesGradeService;
import com.cultural.heritage.service.book.ClothesInfoService;
import jakarta.annotation.Resource;
import org.apache.commons.lang3.ObjectUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.util.List;

@Service
public class ClothesGradeServiceImpl extends ServiceImpl<ClothesGradeMapper, ClothesGrade> implements ClothesGradeService {


    @Resource
    private ClothesInfoService clothesInfoService;


    /**
     * 校验
     */
    @Override
    public void validClothesGrade(ClothesGrade clothesGrade, boolean update) {
        Long id = clothesGrade.getId();
        String clothesType = clothesGrade.getClothesType();
        String image = clothesGrade.getImage();
        BigDecimal minPrice = clothesGrade.getMinPrice();
        BigDecimal maxPrice = clothesGrade.getMaxPrice();
        String brief = clothesGrade.getBrief();

        if (update) {
            if (id == null) {
                throw new BusinessException(ErrorCode.PARAMS_ERROR, "参数id错误");
            }
        }
        if (ObjectUtils.isEmpty(minPrice) || minPrice.compareTo(BigDecimal.ZERO) <= 0) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR, "最低价格参数错误");
        }
        if (ObjectUtils.isEmpty(maxPrice) || maxPrice.compareTo(BigDecimal.ZERO) <= 0) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR, "最高价格参数错误");
        }
        if (minPrice.compareTo(maxPrice) >= 0) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR, "最低价格不能高于最高价格");
        }
        if (StringUtils.isAnyBlank(clothesType, image, brief)) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        QueryWrapper<ClothesGrade> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("clothesType", clothesType);
        if (update) {
            queryWrapper.ne("id", id);
            Long count = this.baseMapper.selectCount(queryWrapper);
            ThrowUtils.throwIf(count > 0, ErrorCode.OPERATION_ERROR, "当前服装等级已存在");
        } else {
            Long count = this.baseMapper.selectCount(queryWrapper);
            ThrowUtils.throwIf(count > 0, ErrorCode.OPERATION_ERROR, "不能重复添加服装等级");
        }
    }


    /**
     * 删除当前服装等级下的所有服装
     */
    @Override
    public void deleteCurrentAllClothes(Long id) {
        ClothesGrade clothesGrade = this.getById(id);
        String clothesType = clothesGrade.getClothesType();
        QueryWrapper<ClothesInfo> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("clothesType", clothesType);
        List<ClothesInfo> list = clothesInfoService.list(queryWrapper);
        // 如果当前服装等级下存在服装，就删掉
        if (!ObjectUtils.isEmpty(list)) {
            boolean result = clothesInfoService.removeBatchByIds(list);
            ThrowUtils.throwIf(!result, ErrorCode.OPERATION_ERROR);
        }
    }
}
